import { Component, ViewChild, HostBinding, Input, HostListener, EventEmitter, Output, forwardRef, Directive, ElementRef } from '@angular/core';
import { DropdownService } from '../dropdown/dropdown.service';
import { SuiDropdownMenu } from '../dropdown/dropdown-menu';
import { NG_VALUE_ACCESSOR } from '@angular/forms';
import { SearchService } from './search.service';
import { readValue } from '../util/util';
export var SuiSearch = (function () {
    function SuiSearch(_element) {
        this._element = _element;
        this.dropdownService = new DropdownService();
        this.searchService = new SearchService();
        this._searchClasses = true;
        this.hasIcon = true;
        this.placeholder = "Search...";
        this.searchDelay = 200;
        this.onItemSelected = new EventEmitter();
    }
    Object.defineProperty(SuiSearch.prototype, "isActive", {
        get: function () {
            return this.dropdownService.isOpen;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(SuiSearch.prototype, "query", {
        get: function () {
            return this.searchService.query;
        },
        set: function (query) {
            var _this = this;
            this.selectedItem = null;
            // Initialise a delayed search.
            this.searchService.updateQueryDelayed(query, function () {
                // Set the results open state depending on whether a query has been entered.
                return _this.dropdownService.setOpenState(_this.searchService.query.length > 0);
            });
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(SuiSearch.prototype, "options", {
        // Sets local or remote options by determining whether a function is passed.
        set: function (options) {
            if (typeof (options) == "function") {
                this.searchService.optionsLookup = options;
                return;
            }
            this.searchService.options = options;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(SuiSearch.prototype, "optionsField", {
        set: function (field) {
            this.searchService.optionsField = field;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(SuiSearch.prototype, "searchDelay", {
        set: function (delay) {
            this.searchService.searchDelay = delay;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(SuiSearch.prototype, "isSearching", {
        get: function () {
            return this.searchService.isSearching;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(SuiSearch.prototype, "results", {
        get: function () {
            return this.searchService.results;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(SuiSearch.prototype, "ngModelChange", {
        // Alias onItemSelected as ngModelChange for [(ngModel)] support.
        get: function () {
            return this.onItemSelected;
        },
        enumerable: true,
        configurable: true
    });
    SuiSearch.prototype.ngAfterViewInit = function () {
        this._menu.service = this.dropdownService;
    };
    // Selects an item.
    SuiSearch.prototype.select = function (item) {
        this.writeValue(item);
        this.onItemSelected.emit(item);
    };
    SuiSearch.prototype.onClick = function (e) {
        e.stopPropagation();
        if (this.searchService.query.length > 0) {
            // Only open on click when there is a query entered.
            this.dropdownService.setOpenState(true);
        }
    };
    // Reads the specified field from an item.
    SuiSearch.prototype.readValue = function (object) {
        return readValue(object, this.searchService.optionsField);
    };
    // Sets a specific item to be selected, updating the query automatically.
    SuiSearch.prototype.writeValue = function (item) {
        this.selectedItem = item;
        this.searchService.updateQuery(item ? this.readValue(item) : "");
    };
    SuiSearch.decorators = [
        { type: Component, args: [{
                    selector: 'sui-search',
                    template: "\n<div class=\"ui input\" [class.icon]=\"hasIcon\">\n    <input class=\"prompt\" type=\"text\" [attr.placeholder]=\"placeholder\" autocomplete=\"off\" [(ngModel)]=\"query\">\n    <i *ngIf=\"hasIcon\" class=\"search icon\"></i>\n  </div>\n<div class=\"results\" suiDropdownMenu menuTransition=\"scale\" menuSelectedItemClass=\"active\">\n    <a class=\"result item\" *ngFor=\"let r of results\" (click)=\"select(r)\">\n        <span *ngIf=\"!searchService.optionsLookup\" [innerHTML]=\"searchService.highlightMatches(r)\"></span>\n        <span *ngIf=\"searchService.optionsLookup\">{{ readValue(r) }}</span>\n    </a>\n    <div *ngIf=\"results.length == 0\" class=\"message empty\">\n        <div class=\"header\">No Results</div>\n        <div class=\"description\">Your search returned no results.</div>\n    </div>\n</div>\n",
                    styles: ["\n/* Ensures results div has margin. */\n:host {\n    display: inline-block;\n}\n\n/* Fixes positioning when results are pushed above the search. */\n.results {\n    margin-bottom: .5em;\n}\n"]
                },] },
    ];
    /** @nocollapse */
    SuiSearch.ctorParameters = function () { return [
        { type: ElementRef, },
    ]; };
    SuiSearch.propDecorators = {
        '_menu': [{ type: ViewChild, args: [SuiDropdownMenu,] },],
        '_searchClasses': [{ type: HostBinding, args: ['class.ui',] }, { type: HostBinding, args: ['class.search',] },],
        'isActive': [{ type: HostBinding, args: ['class.active',] },],
        'hasIcon': [{ type: Input },],
        'placeholder': [{ type: Input },],
        'options': [{ type: Input },],
        'optionsField': [{ type: Input },],
        'searchDelay': [{ type: Input },],
        'isSearching': [{ type: HostBinding, args: ['class.loading',] },],
        'onItemSelected': [{ type: Output },],
        'onClick': [{ type: HostListener, args: ["click", ['$event'],] },],
    };
    return SuiSearch;
}());
// Value accessor for the search.
export var SEARCH_VALUE_ACCESSOR = {
    provide: NG_VALUE_ACCESSOR,
    useExisting: forwardRef(function () { return SuiSearchValueAccessor; }),
    multi: true
};
// Value accessor directive for the search to support ngModel.
export var SuiSearchValueAccessor = (function () {
    function SuiSearchValueAccessor(host) {
        this.host = host;
        this.onChange = function () { };
        this.onTouched = function () { };
    }
    SuiSearchValueAccessor.prototype.writeValue = function (value) {
        this.host.writeValue(value);
    };
    SuiSearchValueAccessor.prototype.registerOnChange = function (fn) {
        this.onChange = fn;
    };
    SuiSearchValueAccessor.prototype.registerOnTouched = function (fn) {
        this.onTouched = fn;
    };
    SuiSearchValueAccessor.decorators = [
        { type: Directive, args: [{
                    selector: 'sui-search',
                    host: {
                        '(onItemSelected)': 'onChange($event)'
                    },
                    providers: [SEARCH_VALUE_ACCESSOR]
                },] },
    ];
    /** @nocollapse */
    SuiSearchValueAccessor.ctorParameters = function () { return [
        { type: SuiSearch, },
    ]; };
    return SuiSearchValueAccessor;
}());
//# sourceMappingURL=search.js.map