import { readValue } from '../util/util';
// T extends JavascriptObject so we can do a recursive search on the object.
export var SearchService = (function () {
    function SearchService(allowEmptyQuery) {
        if (allowEmptyQuery === void 0) { allowEmptyQuery = false; }
        this._options = [];
        // Set default values and reset.
        this.allowEmptyQuery = allowEmptyQuery;
        this.searchDelay = 0;
        this.reset();
    }
    Object.defineProperty(SearchService.prototype, "options", {
        get: function () {
            return this._options;
        },
        set: function (options) {
            this._options = options || [];
            // We cannot use both local & remote options simultaneously.
            this._optionsLookup = null;
            // Reset entire service with new options.
            this.reset();
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(SearchService.prototype, "optionsLookup", {
        get: function () {
            return this._optionsLookup;
        },
        set: function (lookupFn) {
            this._optionsLookup = lookupFn;
            // As before, cannot use local & remote options simultaneously.
            this._options = [];
            this.reset();
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(SearchService.prototype, "optionsField", {
        get: function () {
            return this._optionsField;
        },
        set: function (field) {
            this._optionsField = field;
            // We need to reset otherwise we would now be showing invalid search results.
            this.reset();
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(SearchService.prototype, "results", {
        get: function () {
            return this._results;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(SearchService.prototype, "query", {
        get: function () {
            return this._query;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(SearchService.prototype, "isSearching", {
        get: function () {
            return this._isSearching;
        },
        enumerable: true,
        configurable: true
    });
    // Updates the query after the specified search delay.
    SearchService.prototype.updateQueryDelayed = function (query, callback) {
        var _this = this;
        if (callback === void 0) { callback = function () { }; }
        this._query = query;
        clearTimeout(this._searchDelayTimeout);
        this._searchDelayTimeout = setTimeout(function () {
            _this.updateQuery(query, callback);
        }, this.searchDelay);
    };
    // Updates the current search query.
    SearchService.prototype.updateQuery = function (query, callback) {
        var _this = this;
        if (callback === void 0) { callback = function () { }; }
        this._query = query;
        if (this._query == "" && !this.allowEmptyQuery) {
            // Don't update if the new query is empty (and we don't allow empty queries).
            // Don't reset so that when animating closed we don't get a judder.
            return callback(null);
        }
        if (this._resultsCache.hasOwnProperty(this._query)) {
            // If the query is already cached, make use of it.
            this._results = this._resultsCache[this._query];
            return callback(null);
        }
        if (this._optionsLookup) {
            this._isSearching = true;
            this._optionsLookup(this._query)
                .then(function (results) {
                // Unset 'loading' state, and display & cache the results.
                _this._isSearching = false;
                _this.updateResults(results);
                return callback(null);
            })
                .catch(function (error) {
                // Unset 'loading' state, and throw the returned error without updating the results.
                _this._isSearching = false;
                return callback(error);
            });
            return;
        }
        // Convert the query string to a RegExp.
        var regex = this.toRegex(this._query);
        if (regex instanceof RegExp) {
            // Only update the results if the query was valid regex.
            // This avoids the results suddenly becoming empty if an invalid regex string is inputted.
            this.updateResults(this._options
                .filter(function (o) { return readValue(o, _this._optionsField)
                .toString()
                .match(regex); }));
        }
        return callback(null);
    };
    // Updates & caches the new set of results.
    SearchService.prototype.updateResults = function (results) {
        this._resultsCache[this._query] = results;
        this._results = results;
    };
    // Converts a query string to regex without throwing an error.
    SearchService.prototype.toRegex = function (query) {
        try {
            return new RegExp(query, 'i');
        }
        catch (e) {
            return query;
        }
    };
    // Generates HTML for highlighted match text.
    SearchService.prototype.highlightMatches = function (text) {
        var regex = this.toRegex(this._query);
        if (regex instanceof RegExp) {
            return text.replace(regex, function (match) { return ("<b>" + match + "</b>"); });
        }
        return text;
    };
    // Resets the search back to a pristine state.
    SearchService.prototype.reset = function () {
        this._query = "";
        this._results = [];
        if (this.allowEmptyQuery) {
            this._results = this._options;
        }
        this._resultsCache = {};
        this._isSearching = false;
    };
    return SearchService;
}());
//# sourceMappingURL=search.service.js.map