import { ViewChild, HostBinding, HostListener, Input, ContentChildren } from '@angular/core';
import { DropdownService } from '../dropdown/dropdown.service';
import { SearchService } from '../search/search.service';
import { readValue, KeyCode } from '../util/util';
import { SuiDropdownMenu } from '../dropdown/dropdown-menu';
import { SuiSelectOption } from './select-option';
// We use generic type T to specify the type of the options we are working with, and U to specify the type of the property of the option used as the value.
export var SuiSelectBase = (function () {
    function SuiSelectBase(_element, _renderer) {
        this._element = _element;
        this._renderer = _renderer;
        this.dropdownService = new DropdownService();
        // We do want an empty query to return all results.
        this.searchService = new SearchService(true);
        this.isSearchable = false;
        this.noResultsMessage = "No results";
        this._renderedSubscriptions = [];
        this._selectClasses = true;
    }
    Object.defineProperty(SuiSelectBase.prototype, "isActive", {
        get: function () {
            return this.dropdownService.isOpen;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(SuiSelectBase.prototype, "isVisible", {
        get: function () {
            return this._menu.isVisible;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(SuiSelectBase.prototype, "tabIndex", {
        get: function () {
            // Remove from tabindex if searchable or disabled, as if searchable then the input is what needs to be focussed.
            return this.isSearchable || this.isDisabled ? -1 : 0;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(SuiSelectBase.prototype, "isDisabled", {
        get: function () {
            return this.dropdownService.isDisabled;
        },
        set: function (value) {
            this.dropdownService.isDisabled = !!value;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(SuiSelectBase.prototype, "options", {
        get: function () {
            return this.searchService.options;
        },
        set: function (options) {
            this.searchService.options = options;
            this.optionsUpdateHook();
        },
        enumerable: true,
        configurable: true
    });
    // Hook is here since Typescript doesn't yet support overriding getters & setters while still calling the superclass.
    SuiSelectBase.prototype.optionsUpdateHook = function () { };
    Object.defineProperty(SuiSelectBase.prototype, "availableOptions", {
        get: function () {
            return this.searchService.results;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(SuiSelectBase.prototype, "query", {
        get: function () {
            return this.searchService.query;
        },
        set: function (query) {
            var _this = this;
            this.queryUpdateHook();
            // Update the query then open the dropdown, as after keyboard input it should always be open.
            this.searchService.updateQuery(query, function () {
                return _this.dropdownService.setOpenState(true);
            });
        },
        enumerable: true,
        configurable: true
    });
    // Hook is here since Typescript doesn't yet support overriding getters & setters while still calling the superclass.
    SuiSelectBase.prototype.queryUpdateHook = function () { };
    Object.defineProperty(SuiSelectBase.prototype, "labelField", {
        get: function () {
            return this.searchService.optionsField;
        },
        set: function (field) {
            this.searchService.optionsField = field;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(SuiSelectBase.prototype, "labelGetter", {
        get: function () {
            var _this = this;
            // Helper function to retrieve the label from an item.
            return function (obj) { return readValue(obj, _this.labelField); };
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(SuiSelectBase.prototype, "valueGetter", {
        get: function () {
            var _this = this;
            // Helper function to retrieve the value from an item.
            return function (obj) { return readValue(obj, _this.valueField); };
        },
        enumerable: true,
        configurable: true
    });
    SuiSelectBase.prototype.ngAfterContentInit = function () {
        var _this = this;
        this._menu.service = this.dropdownService;
        // We manually specify the menu items to the menu because the @ContentChildren doesn't pick up our dynamically rendered items.
        this._menu.items = this._renderedOptions;
        // We must call this immediately as changes doesn't fire when you subscribe.
        this.onAvailableOptionsRendered();
        this._renderedOptions.changes.subscribe(function () { return _this.onAvailableOptionsRendered(); });
    };
    SuiSelectBase.prototype.onAvailableOptionsRendered = function () {
        var _this = this;
        // Unsubscribe from all previous subscriptions to avoid memory leaks on large selects.
        this._renderedSubscriptions.forEach(function (rs) { return rs.unsubscribe(); });
        this._renderedSubscriptions = [];
        this._renderedOptions.forEach(function (ro) {
            // Slightly delay initialisation to avoid change after checked errors. TODO - look into avoiding this!
            setTimeout(function () { return _this.initialiseRenderedOption(ro); });
            _this._renderedSubscriptions.push(ro.onSelected.subscribe(function () { return _this.selectOption(ro.value); }));
        });
    };
    SuiSelectBase.prototype.initialiseRenderedOption = function (option) {
        option.usesTemplate = !!this.optionTemplate;
        option.readLabel = this.labelGetter;
        if (option.usesTemplate) {
            this.drawTemplate(option.templateSibling, option.value);
        }
    };
    SuiSelectBase.prototype.selectOption = function (option) {
        // This is implemented individually by the single & multi select variants, but is called within the base class, hence this stub.
        throw new Error("Not implemented");
    };
    SuiSelectBase.prototype.onClick = function (e) {
        e.stopPropagation();
        // Immediately focus the search input whenever clicking on the select.
        this.focusInput();
        // If the dropdown is searchable, clicking should keep it open, otherwise we toggle the open state.
        this.dropdownService.setOpenState(this.isSearchable ? true : !this.dropdownService.isOpen);
    };
    SuiSelectBase.prototype.onKeypress = function (e) {
        if (e.keyCode == KeyCode.Enter) {
            // Enables support for focussing and opening with the keyboard alone.
            this._renderer.invokeElementMethod(this._element.nativeElement, "click");
        }
    };
    SuiSelectBase.prototype.focusInput = function () {
        if (this.isSearchable) {
            // Focusses the search input only when searchable.
            this._renderer.invokeElementMethod(this._queryInput.nativeElement, "focus");
        }
    };
    // Helper that draws the provided template beside the provided ViewContainerRef.
    SuiSelectBase.prototype.drawTemplate = function (siblingRef, value) {
        siblingRef.clear();
        // Use of `$implicit` means use of <template let-option> syntax is supported.
        siblingRef.createEmbeddedView(this.optionTemplate, { '$implicit': value });
    };
    SuiSelectBase.propDecorators = {
        '_menu': [{ type: ViewChild, args: [SuiDropdownMenu,] },],
        '_renderedOptions': [{ type: ContentChildren, args: [SuiSelectOption, { descendants: true },] },],
        '_selectClasses': [{ type: HostBinding, args: ['class.ui',] }, { type: HostBinding, args: ['class.selection',] }, { type: HostBinding, args: ['class.dropdown',] },],
        'isActive': [{ type: HostBinding, args: ['class.active',] },],
        'isVisible': [{ type: HostBinding, args: ['class.visible',] },],
        'isSearchable': [{ type: HostBinding, args: ['class.search',] }, { type: Input },],
        'tabIndex': [{ type: HostBinding, args: ['attr.tabindex',] },],
        'isDisabled': [{ type: HostBinding, args: ['class.disabled',] }, { type: Input },],
        '_queryInput': [{ type: ViewChild, args: ['queryInput',] },],
        'placeholder': [{ type: Input },],
        'options': [{ type: Input },],
        'labelField': [{ type: Input },],
        'valueField': [{ type: Input },],
        'optionTemplate': [{ type: Input },],
        'noResultsMessage': [{ type: Input },],
        'onClick': [{ type: HostListener, args: ["click", ['$event'],] },],
        'onKeypress': [{ type: HostListener, args: ["keypress", ['$event'],] },],
    };
    return SuiSelectBase;
}());
//# sourceMappingURL=select-base.js.map